/*******************************************************************************
*   @file   spi.c
*   @brief  AXI SPI Core Driver.
*   @author ATofan (alexandru.tofan@analog.com)
********************************************************************************
* Copyright 2012(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
********************************************************************************
*   SVN Revision: $WCREV$
*******************************************************************************/

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include "spi.h"
#include "xil_io.h"
#include "xparameters.h"
#include "stdio.h"
#include "stdlib.h"

/******************************************************************************
* @brief Initializes the communication with the Microblaze SPI peripheral.
*
* @return 1
******************************************************************************/
int SPI_Init(void)
{
    int cfgValue = 0;
    
    // Configuration Register Settings
    cfgValue |= (0 << LSBFirst)         | // MSB First transfer format
                (1 << MasterTranInh)    | // Master transactions disabled
                (1 << ManualSlaveAssEn) | // Slave select output follows data in slave select register
                (0 << RxFifoReset)      | // Receive FIFO normal operation
                (0 << TxFifoReset)      | // Transmit FIFO normal operation
                (0 << CHPA)             | // Data valid on first SCK edge
                (0 << CPOL)             | // Active high clock, SCK idles low
                (1 << Master)           | // SPI in Master configuration mode
                (0 << SPE)              | // SPI enabled
                (0 << LOOP);              // Normal operation
                
    // Set corresponding value to the Configuration Register
    Xil_Out32(SPICR, cfgValue);
    return(1);
}

/******************************************************************************
* @brief Pulls Slave Select line Low.
******************************************************************************/
void SPI_SSn_ON(void)
{
	// Enable slave (SSn = 0)
    Xil_Out32(SPISSR, 0x0);
}

/******************************************************************************
* @brief Pulls Slave Select line High.
******************************************************************************/
void SPI_SSn_OFF(void)
{
	// Disable slave (SSn = 1)
    Xil_Out32(SPISSR, 0x1);
}

/******************************************************************************
* @brief Transfers data to and from a SPI slave.
*
* @param spiData - Data to be sent.
* @return spiRead - Data received from SPI Slave device.
******************************************************************************/
int SPI_TransferData(int spiData)
{
    int cfgValue  = 0;
    int spiStatus = 0;
    int spiRead   = 0;
    
    // Write initial data to master SPIDTR register
    Xil_Out32(SPIDTR, spiData);
    
    // Ensure the SPISSR register has all ones
    SPI_SSn_OFF();
    
    // Write configuration data to master SPI device SPICR
    cfgValue |= (0 << LSBFirst)         | // MSB First transfer format
                (1 << MasterTranInh)    | // Master transactions disabled
                (1 << ManualSlaveAssEn) | // Slave select output follows data in slave select register
                (0 << RxFifoReset)      | // Receive FIFO normal operation
                (0 << TxFifoReset)      | // Transmit FIFO normal operation
                (0 << CHPA)             | // Data valid on first SCK edge
                (0 << CPOL)             | // Active high clock, SCK idles low
                (1 << Master)           | // SPI in Master configuration mode
                (1 << SPE)              | // SPI enabled
                (0 << LOOP);              // Normal operation
    Xil_Out32(SPICR, cfgValue);
    
    // Write to SPISSR to manually assert SSn
    SPI_SSn_ON();
    
    // Write same configuration in SPICR but clear inhibit bit
    cfgValue &= ~(1 << MasterTranInh);
    Xil_Out32(SPICR, cfgValue);
    
    // Poll status for completion
    do
    {
        spiStatus = Xil_In32(SPISR);
    }
    while((spiStatus & 0x2) != 2);
    
    // Write all ones to SPISSR
    SPI_SSn_OFF();
    
    // Read received data from SPI Core buffer
    spiRead = Xil_In32(SPIDRR);
    
    return(spiRead);
}

/******************************************************************************
* @brief Generate a delay in miliseconds.
*
* @param ms_count - Number of miliseconds
******************************************************************************/
void delay_ms(u32 ms_count)
{
  u32 count;
  for (count = 0; count < ((ms_count * 100000) + 1); count++) {
    asm("nop");
  }
}

/******************************************************************************
* @brief Print the Degrees/Second.
*
* @param x - Degrees/Second
******************************************************************************/
void PrintDegreesPos(int x)
{
	char c[] = {' ',' ',' ','0','.','0','0','0','0',0};
	int  nr;
	int  i   = 8;
    
	// If the number is in negative degree range
	// set '-' as the beginning character in string
	// and set the corresponding value
	if (x > 32767 * 125)
	{
		nr = (0x10000 * 125) - x;
		c[0] = '-';
	}
    
	// If the number is in positive degree range
	// set '+' as the beginning character in string
	// and set the corresponding value
	else
	{
		nr = x;
		c[0] = '+';
	}
    
	// Extract mod 10 values from the number
	// and place them in the corresponding
	// position in the string, with ASCII conversion
	while(nr > 0)
	{
		c[i] = (nr % 10) + 48;
		nr = nr / 10;
		i = i - 1;
		// Skip the 5th position (it is pre-loaded with '.'
		if(i == 4)
		{
			i = i-1;
		}
	}
    
	// Print the corresponding value on UART
	xil_printf("Degrees / Second = %s\n\r",c);
}
